<?php
/**
 * @package     CSVI
 * @subpackage  Plugin.Joomlacustomfields
 *
 * @author      RolandD Cyber Produksi <contact@rolandd.com>
 * @copyright   Copyright (C) 2006 - @year@ RolandD Cyber Produksi. All rights reserved.
 * @license     GNU/GPLv3 http://www.gnu.org/licenses/gpl-3.0.html
 * @link        https://rolandd.com
 */

defined('_JEXEC') or die;

JLoader::registerPrefix('Rantai', JPATH_SITE . '/administrator/components/com_csvi/rantai');

use Joomla\CMS\Factory;
use Rocloud\Common\File;
use Rocloud\Common\Image;

/**
 * Joomla Custom fields.
 *
 * @package     CSVI
 * @subpackage  Plugin.Joomlacustomfields
 * @since       7.15.0
 */
class PlgCsviextJoomlacustomfields extends RantaiPluginDispatcher
{
	/**
	 * The unique ID of the plugin
	 *
	 * @var    string
	 * @since  7.15.0
	 */
	private $id = 'joomlacustomfields';

	/**
	 * JDatabase handler
	 *
	 * @var    JDatabase
	 * @since  7.15.0
	 */
	protected $db;

	/**
	 * Field Details array
	 *
	 * @var    array
	 * @since  8.3.1
	 */
	protected $fieldDetails = [];

	/**
	 * Construct the class.
	 *
	 * @since   7.15.0
	 */
	public function __construct()
	{
		// Set the dependencies
		$this->db = Factory::getDbo();
	}

	/**
	 * Method to get the field options.
	 *
	 * @param   string             $plugin      The ID of the plugin.
	 * @param   string             $fieldName   The field name
	 * @param   string             $value       The value of field.
	 * @param   int                $itemId      The item ID.
	 * @param   CsviHelperLog      $log         The CSVI logger.
	 * @param   CsviHelperTemplate $template    The template object.
	 *
	 * @return  mixed  Void if plugin needs to handle request | False if plugin does not handle request.
	 *
	 * @since   7.15.0
	 *
	 * @throws  RuntimeException
	 */
	public function importCustomfields($plugin, $fieldName, $value, $itemId, $log, $template)
	{
		if ($plugin !== $this->id)
		{
			return false;
		}

		$jsonValue = false;

		if (strpos($fieldName, '_raw') !== false)
		{
			$jsonValue = true;
			$fieldName = str_replace('_raw', '', $fieldName);
		}

		$fieldResult = $this->getFieldDetails($fieldName);
		$fieldId     = $fieldResult->id;
		$fieldType   = $fieldResult->type;
		$fieldParams = json_decode($fieldResult->fieldparams, true);

		$query = $this->db->getQuery(true)
			->delete($this->db->quoteName('#__fields_values'))
			->where($this->db->quoteName('field_id') . ' = ' . (int) $fieldId)
			->where($this->db->quoteName('item_id') . ' = ' . (int) $itemId);
		$this->db->setQuery($query)->execute();
		$log->add('Removed existing custom field values');

		$importValues = [];
		$processValue = true;

		if (!$value)
		{
			$importValues[] = '';
			$log->add('Empty value imported so clean existing value for field ' . $fieldName);
			$processValue = false;
		}

		if ($processValue)
		{
			switch (strtolower($fieldType))
			{
				case 'repeatable':
					// Value Name#name1|Age#34|Description#My first description~Name#name2|Age#23|Description#My second description
					$importFieldValues = explode('~', $value);

					$fieldCounter = 0;

					foreach ($importFieldValues as $importFieldValue)
					{
						$seperatedFieldValues = explode('|', $importFieldValue);

						foreach ($seperatedFieldValues as $seperatedFieldValue)
						{
							$nextSeperatedFieldValues        = explode('#', $seperatedFieldValue);
							$seperatedFieldName              = $nextSeperatedFieldValues[0];
							$seperatedFieldValue             = $nextSeperatedFieldValues[1];
							$valueArray[$seperatedFieldName] = $seperatedFieldValue;
						}

						$finalArray[$fieldName . $fieldCounter] = $valueArray;
						$fieldCounter++;

					}

					$importValues[] = json_encode($finalArray);
					break;
				case 'subform':
					$customFieldIds = [];
					$finalArray     = [];

					foreach ($fieldParams['options'] as $fieldParam)
					{
						$customFieldIds[] = $fieldParam['customfield'];
					}

					if (!$jsonValue && $value)
					{
						$importFieldValues = explode('~', $value);
						$fieldCounter      = 0;
						$valueArray        = [];
						$importValues      = [];

						foreach ($importFieldValues as $importFieldValue)
						{
							$seperatedFieldValues = explode('|', $importFieldValue);

							foreach ($seperatedFieldValues as $seperatedFieldValue)
							{
								$nextSeperatedFieldValues = explode('#', $seperatedFieldValue);
								$seperatedFieldName       = $nextSeperatedFieldValues[0];
								$seperatedFieldDetails    = $this->getFieldDetails($seperatedFieldName);
								$seperatedFieldId         = $seperatedFieldDetails->id;
								$seperatedFieldValue      = $nextSeperatedFieldValues[1];

								if (is_array($seperatedFieldValue) && strpos($seperatedFieldValue, ':') !== false)
								{
									$seperatedFieldValue = explode(':', $seperatedFieldValue);
								}

								if (in_array($seperatedFieldId, $customFieldIds))
								{
									$valueArray['field' . $seperatedFieldId] = $seperatedFieldValue;
								}
							}

							$finalArray['row' . $fieldCounter] = $valueArray;
							$fieldCounter++;
						}

						if ($finalArray)
						{
							$importValues[] = json_encode($finalArray);
						}
					}
					else
					{
						$importValues[] = $value;
					}

					break;
				case 'acfurl':
				case 'acfsoundcloud':
				case 'acfpaypal':
				case 'acfosm':
					$fieldValues = explode('|', $value);

					foreach ($fieldValues as $fieldValue)
					{
						$eachFieldValues                 = explode('#', $fieldValue);
						$seperatedFieldName              = $eachFieldValues[0];
						$seperatedFieldValue             = $eachFieldValues[1];
						$valueArray[$seperatedFieldName] = $seperatedFieldValue;
					}

					$importValues[] = json_encode($valueArray);
					break;
				case 'acfupload':
					$fieldValues = explode('|', $value);

					if ($fieldValues)
					{
						$files = [];
						foreach ($fieldValues as $key => $fieldValue)
						{
							$fileName    = $fieldValue;
							$title       = '';
							$description = '';

							if (stristr($fieldValue, '#'))
							{
								[$fileName, $title, $description] = explode('#', $fieldValue);
							}

							$key++;
							$files[$key] = [
								'title'       => $title,
								'description' => $description,
								'value'       => $fileName,
							];
						}

						$importValues[] = json_encode($files);
					}
					break;
				case 'media':
					try
					{
						$importValues[] = $this->processImage($value, $template, $log, false);
					}
					catch (Exception $e)
					{
						$log->add($e->getMessage());
						$log->addStats('information', $e->getMessage());
					}

					break;
				case 'imagelist':
					$imageValues = explode('|', $value);

					foreach ($imageValues as $imageValue)
					{
						try
						{
							$importValues[] = $this->processImage($imageValue, $template, $log, true);
						}
						catch (Exception $e)
						{
							$log->add($e->getMessage());
							$log->addStats('information', $e->getMessage());
						}
					}

					break;
				default:
					$multiple = false;

					if (isset($fieldParams['multiple']) && $fieldParams['multiple'])
					{
						$multiple = true;
					}

					// For multiple values value1#value2#value3
					if ((($multiple && strpos($value, '|') !== false)) || strpos($value, '|') !== false)
					{
						$importValues = explode('|', $value);
					}
					else
					{
						$importValues[] = $value;
					}

					break;
			}
		}

		if ($importValues)
		{
			$query->clear()
				->insert($this->db->quoteName('#__fields_values'))
				->columns($this->db->quoteName(array('field_id', 'item_id', 'value')));

			foreach ($importValues as $fieldValues)
			{
				$query->values(
					(int) $fieldId . ',' .
					(int) $itemId . ',' .
					$this->db->quote($fieldValues)
				);
			}

			$this->db->setQuery($query)->execute();
			$log->add('Custom field values added');
		}

		return true;
	}

	/**
	 * Process image fields.
	 *
	 * @param   string $value  Image to process
	 *
	 * @return  mixed  List of values to export if available else empty value.
	 *
	 * @since   8.9.0
	 */
	private function processImage(string $value, $template, $log, $imageList): string
	{
		$imageClass = new Image($log);
		$fileClass  = new File($log);

		$imagePath       = $template->get('file_location_image_files', 'images');
		$customImagePath = !empty($fieldParams['directory']) ? $fieldParams['directory'] : $imagePath;

		if ($template->get('image_field_path'))
		{
			$imagePath = $customImagePath;
		}

		if (substr($imagePath, -1) === '/')
		{
			$imagePath = substr($imagePath, 0, -1);
		}

		$fileClass
			->setBasePath(JPATH_BASE)
			->setDestinationPath($imagePath)
			->setSourceFileName($value)
			->setUserName($template->get('image_username'))
			->setPassword($template->get('image_password'))
			->setMethod($template->get('image_method', 'post'))
			->setCredentialType($template->get('image_credential_type', 'htaccess'))
			->setEncodeUrl($template->get('image_encodeurl', true))
			->setSaveImageOnServer($template->get('save_images_on_server', false))
			->setReDownload($template->get('redownload_external_image', true))
			->setVerifySsl($template->get('image_verify_ssl', true));

		if ($template->get('process_image', false))
		{
			$fileClass
				->setDestinationFileName(basename($value))
				->copyFile();

			$imageClass
				->setSourceFileName($fileClass->getDestinationFile())
				->setDestinationPath($fileClass->getDestinationPath())
				->setDestinationFileName($fileClass->getDestinationFileName())
				->setDestinationExtension($fileClass->getDestinationExtension());

			$fileClass->verifyImageExtension(
				$imageClass->getMimeType()
			);

			if ($template->get('convert_type', 'none') !== 'none')
			{
				$imageClass->setSourceFileName($fileClass->getDestinationFile());
				$imageClass->setDestinationExtension($template->get('convert_type'));
				$imageClass->convertImage();
				$fileClass->setDestinationExtension($template->get('convert_type'));
			}

			if ($template->get('full_resize', 0))
			{
				$imageClass
					->setOutputWidth($template->get('full_width'))
					->setOutputHeight($template->get('full_height'))
					->resize();
			}

			$fileClass->setCase($template->get('change_case', 'none'));
			$imageClass->setSourceFileName($fileClass->getDestinationFile());

			if ($template->get('full_watermark', '0') && $imageClass->isImage())
			{
				$imageClass
					->setSourceFileName($fileClass->getDestinationFile())
					->setWatermarkImage(JPATH_SITE . '/' . $template->get('full_watermark_image'))
					->setMarginRight($template->get('full_watermark_right', 0))
					->setMarginLeft($template->get('full_watermark_bottom', 0))
					->addWatermark();
			}

			if ($imageClass->isImage())
			{
				if ($imageList)
				{
					return $value;
				}

				return $fileClass->getDestinationFile();
			}
		}
	}

	/**
	 * Export the values.
	 *
	 * @param   string        $plugin    The ID of the plugin.
	 * @param   string        $fieldName The field name
	 * @param   string        $value     The value of field.
	 * @param   int           $itemId    The item ID.
	 * @param   CsviHelperLog $log       The CSVI logger.
	 *
	 * @return  mixed  List of values to export if available else empty value.
	 *
	 * @since   7.15.0
	 */
	public function exportCustomfields($plugin, $fieldName, $value, $itemId, $log)
	{
		static $results = [];

		if ($plugin !== $this->id)
		{
			return '';
		}

		$rawFieldname = '';

		if (strpos($fieldName, '_raw') !== false)
		{
			$rawFieldname = $fieldName;
			$fieldName    = str_replace('_raw', '', $fieldName);
		}

		$fieldResult = $this->getFieldDetails($fieldName);
		$fieldId     = $fieldResult->id;
		$fieldType   = $fieldResult->type;
		$hash = md5($fieldId . $itemId);

		if (!isset($results[$hash]))
		{
			$query = $this->db->getQuery(true)
				->select($this->db->quoteName('value'))
				->from($this->db->quoteName('#__fields_values'))
				->where($this->db->quoteName('field_id') . ' = ' . (int) $fieldId)
				->where($this->db->quoteName('item_id') . ' = ' . (int) $itemId);
			$this->db->setQuery($query);
			$results[$hash] = $this->db->loadObjectList();
		}

		$fieldValue = $results[$hash];

		if (!$fieldValue)
		{
			$log->add('No values found for the custom field ' . $fieldName);

			return '';
		}

		$finalArray = [];

		switch (strtolower($fieldType))
		{
			case 'repeatable':
				$storedValues = json_decode($fieldValue[0]->value, true);

				foreach ($storedValues as $storedValue)
				{
					$finalEachArray = [];

					foreach ($storedValue as $key => $customFieldValue)
					{
						$finalEachArray[] = $key . '#' . $customFieldValue;
					}

					$finalArray[] = implode('|', $finalEachArray);
				}

				$value = implode('~', $finalArray);

				break;
			case 'subform':
				$storedValues = json_decode($fieldValue[0]->value, true);

				foreach ($storedValues as $storedValue)
				{
					$finalEachArray = [];

					foreach ($storedValue as $key => $customFieldValue)
					{
						$fieldId = preg_replace('/[^0-9.]+/', '', $key);
						$this->getFieldName($fieldId);
						$fieldName = $this->fieldDetails[$fieldId];

						if (is_array($customFieldValue))
						{
							if (strpos($customFieldValue['imagefile'], '#') !== false)
							{
								$customFieldInnerValues        = explode('#', $customFieldValue['imagefile']);
								$customFieldValue['imagefile'] = $customFieldInnerValues[0];
							}

							$customFieldValue = implode(':', $customFieldValue);
						}

						$finalEachArray[] = $fieldName . '#' . $customFieldValue;
					}

					$finalArray[] = implode('|', $finalEachArray);
				}

				$value = implode('~', $finalArray);

				if ($rawFieldname)
				{
					$value = $fieldValue[0]->value;
				}

				break;
			case 'acfurl':
			case 'acfsoundcloud':
			case 'acfpaypal':
			case 'acfosm':
				$storedValues = json_decode($fieldValue[0]->value, true);

				foreach ($storedValues as $key => $customFieldValue)
				{
					$splitValues[] = $key . '#' . $customFieldValue;
				}

				$value = implode('|', $splitValues);

				break;
			case 'acfupload':
				$storedValues = json_decode($fieldValue[0]->value, true);

				foreach ($storedValues as $key => $customFieldValue)
				{
					$splitValues[] = $customFieldValue['value'] . '#' . $customFieldValue['title'] . '#' . $customFieldValue['description'];
				}

				$value = implode('|', $splitValues);

				break;
			default:
				$finalArray = [];

				foreach ($fieldValue as $fieldVal)
				{
					$finalArray[] = $fieldVal->value;
				}

				$value = implode('|', $finalArray);
				break;
		}

		return $value;
	}

	/**
	 * Get custom field Details
	 *
	 * @param   string  $fieldName  The field name
	 *
	 * @return  array  List of field values
	 *
	 * @since   7.15.0
	 */
	private function getFieldDetails($fieldName)
	{
		$query = $this->db->getQuery(true)
			->select(
				$this->db->quoteName(
					[
						'id',
						'type',
						'fieldparams'
					]
				)
			)
			->from($this->db->quoteName('#__fields'))
			->where($this->db->quoteName('name') . '  = ' . $this->db->quote($fieldName));
		$this->db->setQuery($query);

		return $this->db->loadObject();
	}

	/**
	 * Get custom field name from ID
	 *
	 * @param   int  $fieldId  The field ID
	 *
	 * @return  string
	 *
	 * @since   8.3.1
	 */
	private function getFieldName(int $fieldId): string
	{
		if (!array_key_exists($fieldId, $this->fieldDetails))
		{
			$query = $this->db->getQuery(true)
				->select(
					$this->db->quoteName('name'))
				->from($this->db->quoteName('#__fields'))
				->where($this->db->quoteName('id') . '  = ' . $fieldId);

			$this->db->setQuery($query);
			$this->fieldDetails[$fieldId] = $this->db->loadResult();
		}

		return $this->fieldDetails[$fieldId];
	}
}
